
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General
# Public License along with this library; if not, write to the
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


################################################################################
# FUNCTION:                         DESCRIPTION:
#  getWindows                        Extracts windows information             
#   getWindowsFun                     Extracts windows function
#   getWindowsParams                  Extracts windows function parameters        
#   getWindowsHorizon                 Extracts windows Horizon
# FUNCTION:                         DESCRIPTION:
#  getStrategy                       Extracts strategy information
#   getStrategyFun                    Extracts strategy function
#   getStrategyParams                 Extracts strategy function parameters
# FUNCTION:                         DESCRIPTION:
#  getSmoother                       Extracts smoother information
#   getSmootherFun                    Extracts smoother function
#   getSmootherParams                 Extracts smoother function parameters
#   getSmootherLambda                 Extracts decay parameter
#   getSmootherDoubleSmoothing        Extracts double smoothing flag
#   getSmootherInitialWeights         Extracts initial weights
#   getSmootherSkip                   Extracs skip smoothing flag
# FUNCTION:                         DESCRIPTION:
#  getMessages                       Extracts messages
################################################################################


getWindows <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getWindows")
}


# ------------------------------------------------------------------------------


getWindowsFun <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getWindowsFun")
}


# ------------------------------------------------------------------------------


getWindowsParams <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getWindowsParams")
}


# ------------------------------------------------------------------------------


getWindowsHorizon <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getWindowsHorizon")
}


# ------------------------------------------------------------------------------


getSmoother <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmoother")
}


# ------------------------------------------------------------------------------


getSmootherFun <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherFun")
}

# ------------------------------------------------------------------------------


getSmootherParams <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherParams")
}


# ------------------------------------------------------------------------------


getSmootherLambda <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherLambda")
}

# ------------------------------------------------------------------------------


getSmootherDoubleSmoothing <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherDoubleSmoothing")
}


# ------------------------------------------------------------------------------


getSmootherInitialWeights <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherInitialWeights")
}


# ------------------------------------------------------------------------------

getSmootherSkip <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getSmootherSkip")
}

# ------------------------------------------------------------------------------


getStrategy <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getStrategy")
}


# ------------------------------------------------------------------------------


getStrategyFun <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    UseMethod("getStrategyFun")
}


# ------------------------------------------------------------------------------


getStrategyParams <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getStrategyParams")
}


# ------------------------------------------------------------------------------


if(FALSE) {
# Already defined in fPortfolio
getMessages <-
function(object)
{   
    # A function implemented by Diethelm Wuertz and William Chen
    
    # Description:
    
    # Arguments:
    
    # FUNCTION:
    
    # Return Value:
    UseMethod("getMessages")
}
}


################################################################################

