
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General 
# Public License along with this library; if not, write to the 
# Free Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
# MA  02111-1307  USA


###############################################################################
# GLOBAL SETTINGS:         DESCRIPTION: 
#  Data                     Defines global portfolio Data object
#  portfolioObjective       Defines global portfolio objective function
#  portfolioReturn          Defines global portfolio return function
#  portfolioRisk            Defines global portfolio risk function
###############################################################################


# Global Variables, do it properly ...


# -----------------------------------------------------------------------------


Data <- NA


# -----------------------------------------------------------------------------


portfolioObjective <- 
  function(weights) 
{ 
  NA 
} 


# -----------------------------------------------------------------------------


portfolioReturn <- 
  function(weights) 
{ 
  NA 
}


# -----------------------------------------------------------------------------


portfolioRisk <- 
  function(weights) 
{ 
  NA 
} 


###############################################################################


